package com.bursie.harvester.logic;

import com.bursie.harvester.sources.WordSource;
import com.bursie.harvester.sources.WordnetSource;
import com.bursie.harvester.sources.YahooSource;
import java.util.*;

/**
 * A class which keeps much of the logic for the application: it uses the word sources and checks the relevance
 * of the words given by the sources.
 */
public class KeywordExpert {

    private RelevanceChecker relevanceChecker;
    private StopWords stopWords;
    private List<WordSource> wordSources;

    public KeywordExpert(List<WordSource> wordSources, RelevanceChecker relevanceChecker, StopWords stopWords) {
        this.wordSources = wordSources;
        this.relevanceChecker = relevanceChecker;
        this.stopWords = stopWords;
    }

    /**
     * Returns the default KeywordExpert with all known word sources
     * @return The default KeywordExpert with all known word sources
     */
    public static KeywordExpert getDefaultKeywordExpert() {
        List<WordSource> sources = new ArrayList<WordSource>();
        sources.add(new WordnetSource());
        sources.add(new YahooSource());
        return new KeywordExpert(sources, new RelevanceChecker(), new StopWords());
    }

    /**
     * Find keywords related to the search keyword
     *
     * @param seedKeyword A seed keyword for which we search related keywords
     * @return An ordered list of keywords; the most relevant keyword is presented first. Each keyword only occurs once.
     */
    public List<Word> getSuggestedKeywords(String seedKeyword) {
        if (seedKeyword == null) {
            return null;
        }

        seedKeyword = stopWords.removeStopWords(seedKeyword);

        List<Word> keyWords = new ArrayList<Word>();
        Set<String> suggestedKeywords = new HashSet<String>();
        for (WordSource wordSource : wordSources) {
            suggestedKeywords.addAll(wordSource.getSuggestedKeywords(seedKeyword));
        }

        List<Word> words = relevanceChecker.checkRelevance(seedKeyword, suggestedKeywords);

        keyWords.addAll(words.subList(0, Math.min(30, words.size()))); // the 30 most interesting words, if there are that many
        Collections.sort(keyWords);
        return keyWords;
    }
}
