package com.bursie.harvester.sources;

import com.bursie.harvester.util.InternetQuery;
import com.google.gson.FieldNamingPolicy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonParseException;
import java.io.IOException;
import java.util.Collections;
import java.util.List;

/**
 * A WordSource that uses Yahoo related search internet query for finding related keywords.
 */
public class YahooSource implements WordSource {
    private static final String YAHOO_URL_BASE = "http://search.yahooapis.com/WebSearchService/V1/relatedSuggestion?appid=YahooDemo&output=json&query=";

    private Gson gson;
    private InternetQuery internetQuery;

    public YahooSource() {
        this(new GsonBuilder().setFieldNamingPolicy(FieldNamingPolicy.UPPER_CAMEL_CASE).create(), new InternetQuery());
    }

    public YahooSource(Gson gson, InternetQuery internetQuery) {
        this.gson = gson;
        this.internetQuery = internetQuery;
    }

    /**
     * Finds related keywords given a seed keyword
     *
     * @param seedKeyword A seed keyword, doesn't need to be URL escaped.
     * @return A List of related keywords. Might contain duplicates!
     */
    public List<String> getSuggestedKeywords(String seedKeyword) {
        try {
            // get a json formatted String using an internet query
            String json = internetQuery.getQueryResult(YAHOO_URL_BASE, seedKeyword);

            // for an empty result, yahoo returns "\n" instead of the json standard "null" object. Gson can't handle it
            if ("{\"ResultSet\":\"\\n\"}".equals(json)) {
                return Collections.emptyList();
            }

            // get suggested keywords from json using gson
            GsonYahooObject gsonYahooObject = gson.fromJson(json, GsonYahooObject.class);
            List<String> words = gsonYahooObject.getResultSet().getResult();
            return words;

        } catch (IOException e) {
            e.printStackTrace();
            return Collections.emptyList();
        } catch (JsonParseException e) {
            e.printStackTrace();
            return Collections.emptyList();
        } catch (ClassCastException e) {
            e.printStackTrace();
            return Collections.emptyList();
        }
    }

    /**
     * The class for the main object returned by the Yahoo request
     */
    private static class GsonYahooObject {
        private YahooResultSet resultSet;

        public YahooResultSet getResultSet() {
            return resultSet;
        }
    }

    /**
     * The class for the individual results (related keywords)
     */
    private static class YahooResultSet {
        private List<String> result;

        public List<String> getResult() {
            return result;
        }
    }
}
